/** @(#) listpos/dynamiclistpos.cpp */
/**
* Implementa la lista con almacenamiento dinamico.
* @author Omar Posada Villarreal
* @version 2.0, 10/12/2001 C++
* @version 1.1, 15/10/2001 getFirst(), getNext()
* @version 1.0, 03/10/2001
*/
#include "dynamiclistpos.h"

// constructor, ~, friend-----------------------------------------------------

/** Constructor. Inicializa a "first" y "last". */
template <class TObj>
DynamicListPos<TObj> :: DynamicListPos() : ListPos<TObj>() {
	//cout << "{ dlp";
        itemCount = 0;
	first = last = NULL;
}

/** Destructor. Libera nodos. */
template <class TObj>
DynamicListPos<TObj> :: ~DynamicListPos() {
	removeAllElements();
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) y separado por espacios.
* @return Flujo de los elementos.
*/
// oper unario friend no hace referencia a la clase
/* error Linker template <class TObj>
ostream& operator<< (ostream &outStr, const DynamicListPos<TObj> &list) { // 2 const
	ListNodePos<TObj>	*pCurrent = list.getFirstNode();
	outStr << "";	// inicializar

	if ( list.isEmpty() ) {
		return outStr;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
		//sList += pCurrent->item;
		outStr << pCurrent->getItem();
		pCurrent = pCurrent->getNext();
	}
	return outStr;
}
*/

// public---------------------------------------------------------------------

/**
* Cuenta el numero de elementos en la lista.
* @return Numero entero de elementos.
*/
template <class TObj>
int DynamicListPos<TObj> :: size() const {
	// error int DynamicListPos :: size() {
	return itemCount;
}

/**
* Checa si la lista esta vacia.
* @return true No hay elementos en la lista
*/
template <class TObj>
bool DynamicListPos<TObj> :: isEmpty() const {
	// Si el primero apunta a NULL, se considera vacio,
	// No se checa a last
	return (first == NULL);
}

/**
* Checa si la lista esta llena.
* En teoria, la lista puede crecer indefinidamente,
* pero para conservar la consistencia con "size()" se limita
* al maximo soportado por un entero.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool DynamicListPos<TObj> :: isFull() const {
	return (itemCount >= INT_MAX_VALUE);
}


/**
* Elimina todos los elementos de la lista. Libera memoria.
* No lanza "Exception".
*/
template <class TObj>
void DynamicListPos<TObj> :: removeAllElements() {
	// Mientras haya nodos, quitarlos
	while ( ! isEmpty() ) {
        	removeFirst();	// mas rapido que removeLast()
	}
	// redundante first = last = NULL; // no liberan memo
}

/**
* Inserta un elemento al inicio de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void DynamicListPos<TObj> :: addFirst(const TObj item)
		throw (FullListExceptionPos) {
	if (isEmpty()) {
		// No usar first = ListNodePos(); last = ListNodePos();
		// ya que se crearian dos copias sin relacion;
		first = last = new ListNodePos<TObj>(item);
	} else {
		first = new ListNodePos<TObj>(item, first);
	}
        ++itemCount;
}

/**
* Inserta un elemento al final de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void DynamicListPos<TObj> :: addLast(const TObj item)
		throw (FullListExceptionPos) {
	if (isEmpty()) {
		first = last = new ListNodePos<TObj>(item);
	} else {
                last->setNext( new ListNodePos<TObj>(item) );
		last = last->getNext();
                //cout << "\nlast: " << last; // log
	}
        ++itemCount;
}

/**
* Toma el elemento al inicio de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DynamicListPos<TObj> :: removeFirst() throw(EmptyListExceptionPos) {
	// Error comun: olvidar poner "new throw Exception"
	TObj removeItem;	// no usar = NULL;

	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	removeItem = first->getItem();	// getItem();

	// Restablecer referencias de first y last
        ListNodePos<TObj> *aux = first;	// delete
	if (first == last) {
		first = last = NULL;
	} else {
		first = first->getNext();	// getNext();
	}
        delete aux;
        --itemCount;
	return removeItem;
}

/**
* Toma el elemento al final de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DynamicListPos<TObj> :: removeLast() throw(EmptyListExceptionPos) {
	TObj removeItem; // No inicializar con = NULL si es string;
	ListNodePos<TObj> *pCurrent;

	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	removeItem = last->getItem();	// getItem();

	// Restablecer referencias de first y last
        ListNodePos<TObj> *aux = last;	// delete
	if (first == last) {
		first = last = NULL;
	} else {
		pCurrent = first;
		// Recorre la lista
		while (pCurrent->getNext() != last) {
			pCurrent = pCurrent->getNext();
		}
		// El ultimo apunta al actual y se termina la lista
		last = pCurrent;
		pCurrent->setNext(NULL);
	}
        delete aux;
        --itemCount;
	return removeItem;
}

/**
* Regresa el contenido del elemento al final de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DynamicListPos<TObj> :: getLast() throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return last->getItem();
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "TObj.toString()" y separado por espacios.
*/
// @return Cadena con los elementos
template <class TObj>
void DynamicListPos<TObj> :: print() const {
	//string	sList("");
	ListNodePos<TObj>	*pCurrent = first;

	if ( isEmpty() ) {
		return;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
		//sList += pCurrent->item;
		cout << pCurrent->getItem() << SEPARATOR;
		pCurrent = pCurrent->getNext();
	}
	//return sbList.toString();
}

/**
* Regresa el contenido del elemento al inicio de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DynamicListPos<TObj> :: getFirst() throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return first->item;
}

/**
* Regresa el primer nodo de la lista
* sin eliminarlo.
* @return Primer ListNodePos.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
ListNodePos<TObj> *  DynamicListPos<TObj> :: getFirstNode()
		throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return first;
}

/**
* Regresa el contenido del siguiente elemento de la lista
* sin eliminarlo.
* @param node Nodo del que se tomara el siguiente.
* @return ListNodePos con el nodo siguiente.
* @throw EmptyListExceptionPos La lista esta vacia.
* El nodo es NULL.
*/
template <class TObj>
ListNodePos<TObj> *  DynamicListPos<TObj> :: getNextNode(
		const ListNodePos<TObj> * node)
		throw(EmptyListExceptionPos){
	if (node == NULL) {
		throw EmptyListExceptionPos("El nodo es NULL.");
	}
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return node->next;
}

/**
* Regresa el contenido del elemento actual de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DynamicListPos<TObj> :: get(const ListNodePos<TObj> * node)
		throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return node->item;
}

// private--------------------------------------------------------------------
// Fin------------------------------------------------------------------------
