/** @(#) listpos/queuepos.cpp */
/**
* Implementa los metodos comunes a las colas.
* Representacion "LIFO", ultimo en entrar, primero en salir 
* ("Last in, first out").
* Usa las implementaciones de "ListPos".
* @author Omar Posada Villarreal
* @version 2.0, 10/12/2001 C++
* @version 1.0, 03/10/2001
*/

#include "queuepos.h"

// constructor, ~, friend-----------------------------------------------------
/**
* Instancia o crea la lista a manejar.
* Uso: pTheList = new DynamicListPos();
* @param pTheList Lista concreta de un cierto tipo.
*/
template <class TObj>
QueuePos<TObj> :: QueuePos(ListPos<TObj> *pTheList) {
	pList = pTheList;
}

// Destructor predeterminado. La lista libera memoria

// public---------------------------------------------------------------------
/**
* Cuenta el numero de elementos en la cola.
* @return Numero entero de elementos.
*/
template <class TObj>
int QueuePos<TObj> :: size() const {
	return pList->size();
}

/**
* Checa si la cola esta vacia.
* @return true No hay elementos en la cola
*/
template <class TObj>
bool QueuePos<TObj> :: isEmpty() const {
	return pList->isEmpty();
}

/**
* Checa si la cola esta llena.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool QueuePos<TObj> :: isFull() const {
	return pList->isFull();
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "Object.print()" y separado por espacios.
*/
template <class TObj>
void QueuePos<TObj> :: print() const {
	pList->print();
}

/** Elimina todos los elementos de la cola. No lanza "Exception". */
template <class TObj>
void QueuePos<TObj> :: removeAllElements() {
	pList->removeAllElements();
}

/**
* Forma en la cola. Encola un nuevo elemento al final.
* @param item Elemento a agregar al final de la cola
* @throws FullListExceptionPos La fila no tiene mas capacidad
*/
template <class TObj>
void QueuePos<TObj> :: queue(TObj item) throw (FullListExceptionPos) {
	pList->addLast(item);
}


/**
* Retira del inicio de la cola. Desencola el primer elemento.
* Uso: <pre>
*	itemString = (String)myQueue.dequeue();
* </pre>
* @return item Elemento retirado.
* @throws EmptyListExceptionPos La fila esta vacia.
*/
template <class TObj>
TObj QueuePos<TObj> :: dequeue() throw(EmptyListExceptionPos) {
	return pList->removeFirst();
}

// private--------------------------------------------------------------------
// Fin------------------------------------------------------------------------

