/** @(#) listpos/setpos.cpp */
/**
* Implementa el TDA de conjuntos con operaciones basicas.
* El conjunto es una lista para las operaciones de referencia
* y de insercion/eliminacion de una lista.
* @author Omar Posada Villarreal
* @version 2.0, 03/01/2002 C++
* @version 1.0, 28/10/2001
*/
#include "setpos.h"

// constructor, ~, friend-----------------------------------------------------
/**
* Construye un conjunto a partir de un arreglo. NULL es el conjunto vacio.
* Se puede tener un conjunto con un elemento nulo.
* Del contador se encarga la lista.
* @param theSet Representacion del conjunto.
* @param count Numero de elementos.
* @throws FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
SetPos<TObj> :: SetPos(TObj theSet[], int count) throw (FullListExceptionPos) {
	// :  DLinkListPos() == super();
	//	setStringSet((string[])theSet);
        setSet(theSet, count);
}

/**
* Construye un conjunto sin elementos.
* NULL es el conjunto vacio.
*/
template <class TObj>
SetPos<TObj> :: SetPos() {
}

/**
* Construye un conjunto copiando los elementos de otro.
* NULL es el conjunto vacio.
* @param theSet Conjunto del que se copiaria la informacion.
*/
template <class TObj>
SetPos<TObj> :: SetPos(const SetPos<TObj> &theSet) {
	DLinkNodePos<TObj>	*pCurrent;
        TObj	item;

        pCurrent = theSet.first;	// antes getFirstNode();
	if ( theSet.isEmpty() ) {
		return;
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
	        item = pCurrent->getItem();
                this->addLast(item);
		pCurrent = pCurrent->getRight();
	}
}
/* Destructor predeterminado. */

// operator--------------------------------------------------------------------

/**
* Salida en un renglon a los elementos del conjunto.
* Formato: { hola, mundo }.
* @param outStr Flujo de salida.
* @param theSet Conjunto.
* @return Flujo de salida con la lista.
*/
/*template <class TObj>
ostream & operator<<(ostream &outStr, const SetPos<TObj> &theSet) {
	SetPos<TObj>	*pCurrent;
        TObj	item;
	outStr << "{ ";	// inicializar

        pCurrent = theSet.getFirstNode();
	if ( theSet.isEmpty() ) {
	        outStr << " }";
		return outStr;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
	        item = pCurrent->getItem();
		outStr << item << ", ";
		pCurrent = pCurrent->getRight();
	}
        outStr << " }";
	return outStr;
}
*/

// public---------------------------------------------------------------------

/**
* Entrada de conjuntos.
* @param name Nombre del conjunto.
* @return Numero de elementos.
*/
template <class TObj>
int SetPos<TObj> :: input() {
	TObj 	item;
        int	count;
        cout << "\nNumero de elementos: ";
        cin >> count;
        cout << "Entrada de elementos.\n";
        for (int i = 0; i < count; ++i) {
	        cout << "\t[" << i << "]: ";
	        cin >> item;
                this->addLast(item);
        }
        return count;
}
/**
* Salida en un renglon a los elementos del conjunto.
* Formato: { hola, mundo }.
*/
template <class TObj>
void SetPos<TObj> :: print() {
	DLinkNodePos<TObj>	*pCurrent;
        TObj	item;
	cout << "{ ";	// inicializar

        pCurrent = this->first;	// antes getFirstNode();
	if ( this->isEmpty() ) {
	        cout << " }";
		return;
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
	        item = pCurrent->getItem();
		cout << item;
		pCurrent = pCurrent->getRight();
                if (pCurrent != NULL) {
                	cout << ", ";
                }
	}
        cout << " }";
}

/**
* Eliminar repeticiones de contenidos.
* @return Numero de repeticiones eliminadas.
*/
template <class TObj>
int SetPos<TObj> :: removeDuplicates() {
	DLinkNodePos<TObj>	*pNode;
	DLinkNodePos<TObj>	*pFound;
	TObj obj;
	int		rep = 0;	//repetitions;

	if (this == NULL) {
		return 0;
	}

	pNode = this->first;	// this para claridad
	while (pNode != NULL) {
		pFound = this->find(pNode->getItem() );
		// Si el encontrado es distinto del actua, es duplicado
		if (pFound != pNode) {
			rep++;
			remove(pNode);
		}
		pNode = pNode->getRight();
	}
	return rep;
}

/**
* Convierte un conjunto de objetos en un conjunto.
* @param theSet Arreglo de objetos.
* @return Numero de repeticiones eliminadas.
*/
template <class TObj>
int SetPos<TObj> :: setSet(TObj theSet[], int count)
		throw (FullListExceptionPos) {
	TObj	obj;

	if ((theSet == NULL) || (count < 1)) {
		return 0;
	}

	// Agregar elementos
	for (int i = 0; i < count; i++) {
		obj = theSet[i];
		// No se puede para numeros if ( !str.equals("") ) { }
                addLast(obj);
	}
	return removeDuplicates();
}

/**
* Elementos que pertenecen al conjunto actual, a setB o a ambos conjuntos.
* Agrega los elementos de setB al final.
* @param setB Conjunto a unir con el actual.
* @return Conjunto union.
*/
template <class TObj>
SetPos<TObj> * SetPos<TObj> :: unite(const SetPos<TObj> &setB) {
	// No usar union, es palabra reservada.
        // Solo puede funcionar con apuntadores a new,
        // porque al terminar las var. locales son destruidas.

	// Copia los elementos al resultado
	SetPos<TObj> *pA;
	SetPos<TObj> *pB;

	// Si uno es vacio, crea y regresa el otro
	if (this->isEmpty() ) {
        	pB = new SetPos(setB);
		return pB;
	}

	if (setB.isEmpty() ) {
        	pA = new SetPos(*this);
		return pA;
	}

        // Ambos tienen elementos
        pA = new SetPos(*this);
        pB = new SetPos(setB);
	DLinkNodePos<TObj>	*lastA;
	DLinkNodePos<TObj>	*firstB;

	// Unir ultimo de A con primero de B
	lastA 	= pA->last;	// antes getLastNode();
	firstB 	= pB->first;	// antes getFirstNode();

	lastA->setRight(firstB);
	firstB->setLeft(lastA);

	//a.setLastNode(b.getLastNode());	//Actualizar first last
	pA->last = pB->last;	// era getLastNode();	//Actualizar first last

	pA->removeDuplicates();

	//return this;
        return pA;	// debe regresar un apuntador
}

/**
* Elementos que pertenecen tanto al actual como a setB.
* @param setB Conjunto a unir con el actual.
* @return conjunto interseccion.
*/
template <class TObj>
SetPos<TObj> * SetPos<TObj> :: intersection(const SetPos<TObj> &setB)
		throw (FullListExceptionPos) {
        // Solo puede funcionar con apuntadores a new,
        // porque al terminar las var. locales son destruidas.
	SetPos<TObj>	*pRes = new SetPos();	//conjunto vacio

	// Si uno es vacio, regresa conjunto vacio
	if (this->isEmpty() ) {
		return pRes;
	}

	if (setB.isEmpty() ) {
		return pRes;
	}

	// Si se encuentra en ambos conjuntos, agregarlo
	DLinkNodePos<TObj>	*pNodeA;
	TObj	elem;

	pNodeA	= this->first;
	do {
		elem = pNodeA->getItem();
		if ( setB.find(elem) != NULL) {
			pRes->addLast(elem);
		}

		pNodeA = pNodeA->getRight();
	} while (pNodeA != NULL);

	pRes->removeDuplicates();

	return pRes;
}

/**
* Elementos que pertenecen tanto al actual pero no a setB.
* @param setB Conjunto a quitar del actual.
* @return Conjunto diferencias.
*/
template <class TObj>
SetPos<TObj> * SetPos<TObj> :: difference(const SetPos<TObj> &setB)
		throw (FullListExceptionPos) {
        // Solo puede funcionar con apuntadores a new,
        // porque al terminar las var. locales son destruidas.
	SetPos<TObj>	*pRes = new SetPos();	//conjunto vacio

	// Si este es vacio, no se le puede quitar nada
	if (this->isEmpty() ) {
		return pRes;
	}

	// Si no se le quita nada, queda igual
	// Si se encuentra en ambos conjuntos, agregarlo
	if (setB.isEmpty() ) {
	        delete pRes;
        	pRes = new SetPos(*this);
		return pRes;
	}

	DLinkNodePos<TObj>	*pNodeA;
	TObj	elem;

	// Si se encuentra en setB, se elimina
	pNodeA	= first;
	do {
		elem = pNodeA->getItem();
		if (setB.find(elem) == NULL ) {
			pRes->addLast(elem);
		}

		pNodeA = pNodeA->getRight();
	} while (pNodeA != NULL);

	// Quita repeticiones
	pRes->removeDuplicates();

	return pRes;
}
// Fin------------------------------------------------------------------------




